import { TextChannel } from 'eris';
import { v4 as genUUID } from 'uuid';
import { RichEmbed, Route, Server } from '../../../class';

export default class Root extends Route {
  constructor(server: Server) {
    super(server);
    this.conf = {
      path: '/',
    };
  }

  public bind() {
    this.router.post('/executive-orders/new', async (req, res) => {
      if (!req.body.pin) {
        return res.status(401).json({
          code: this.constants.codes.UNAUTHORIZED,
          message: this.constants.messages.UNAUTHORIZED,
        });
      }

      const director = await this.server.client.db.Score.findOne({ pin: req.body.pin });

      if (!director) {
        return res.status(401).json({
          code: this.constants.codes.UNAUTHORIZED,
          message: this.constants.messages.UNAUTHORIZED,
        });
      }

      if (!req.body.subject) {
        return res.status(400).json({
          code: this.constants.codes.CLIENT_ERROR,
          message: this.constants.messages.CLIENT_ERROR,
        });
      }

      if (!req.body.body) {
        return res.status(400).json({
          code: this.constants.codes.CLIENT_ERROR,
          message: this.constants.messages.CLIENT_ERROR,
        });
      }

      const executiveOrder = await this.server.client.db.ExecutiveOrder.create({
        issuedBy: director.userID,
        subject: req.body.subject,
        body: req.body.body,
        at: new Date(),
        oID: genUUID(),
      });

      const staffInformation = await this.server.client.db.Staff.findOne({ userID: director.userID });
      const staffDiscord = this.server.client.users.get(director.userID) || await this.server.client.getRESTUser(director.userID);

      const embed = new RichEmbed();
      embed.setTitle('Executive Order');
      embed.setAuthor(`${staffDiscord.username}#${staffDiscord.discriminator}, ${staffInformation.pn.join(', ')}`, staffDiscord.avatarURL);
      embed.setColor('#dd3acd');
      embed.addField('Subject', req.body.subject);
      embed.addField('Body', req.body.body);
      embed.addField('ID', executiveOrder.oID);
      embed.setTimestamp(new Date());

      const channel = <TextChannel>this.server.client.getChannel('807444198969835550');
      await channel.createMessage({ embed });

      return res.status(200).json({
        code: this.constants.codes.SUCCESS,
        message: `Created new Executive Order with ID ${executiveOrder.oID} by the ${staffDiscord.username}#${staffDiscord.discriminator}, ${staffInformation.pn.join(', ')}.`,
      });
    });
  }
}
